from ftw.avatar.default import DefaultAvatarGenerator
from ftw.avatar.testing import AVATAR_FUNCTIONAL_TESTING
from ftw.builder import Builder
from ftw.builder import create
from PIL import Image
from StringIO import StringIO
from unittest2 import TestCase
from zope.component.hooks import getSite


class TestDefaultAvatarGenerator(TestCase):
    layer = AVATAR_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestDefaultAvatarGenerator, self).setUp()
        self.portal_membership = getSite().portal_membership

    def test_generates_220x220_image(self):
        user = create(Builder('user').named('Hugo', 'Boss'))
        output = StringIO()
        DefaultAvatarGenerator().generate(user.getId(), output)
        output.seek(0)
        self.assertEquals((220, 220), Image.open(output).size)

    def test_text_is_capital_first_letters_of_first_two_words(self):
        user = create(Builder('user').named('Foo', 'Bar'))
        user2 = create(Builder('user').named('Foo', 'Goo Poo'))
        self.assertEquals('BF', DefaultAvatarGenerator().text(user.getId()))
        self.assertEquals('GP', DefaultAvatarGenerator().text(user2.getId()))

    def test_text_is_two_letters_of_word_when_only_one_word_given(self):
        user = create(Builder('user').named('Foo', 'Bar'))
        member = self.portal_membership.getMemberById(user.getId())
        member.setMemberProperties(mapping={"fullname": "Foo"})
        self.assertEquals('FO', DefaultAvatarGenerator().text(user.getId()))

    def test_text_is_only_one_letter(self):
        user = create(Builder('user').named('Foo', 'Bar'))
        member = self.portal_membership.getMemberById(user.getId())
        member.setMemberProperties(mapping={"fullname": "x"})
        self.assertEquals('X', DefaultAvatarGenerator().text(user.getId()))

    def test_text_is_dash_if_no_names_defined(self):
        self.assertEquals('-', DefaultAvatarGenerator().text(None))
        self.assertEquals('-', DefaultAvatarGenerator().text(''))

    def test_foreground_color_is_white_by_default(self):
        self.assertEquals((255, 255, 255),
                          DefaultAvatarGenerator().foreground_color())

    def test_background_color_is_valid_color_tuple(self):
        self.assertEquals(
            [int, int, int],
            map(type, DefaultAvatarGenerator().background_color()))

    def test_text_with_leading_and_trailing_spaces(self):
        user = create(Builder('user').named('Foo', 'Bar'))
        member = self.portal_membership.getMemberById(user.getId())
        member.setMemberProperties(mapping={"fullname": " Hugo Boss "})
        self.assertEquals('HB', DefaultAvatarGenerator().text(user.getId()))

    def test_text_of_user_without_fullname_is_userid(self):
        self.assertEquals('AD', DefaultAvatarGenerator().text('admin'))

    def test_text_of_non_existent_user(self):
        self.assertEquals('UN', DefaultAvatarGenerator().text('unkown.user'))

    def test_text_of_non_existent_user_with_leading_spaces(self):
        self.assertEquals('UU', DefaultAvatarGenerator().text('  unkown user  '))



from ftw.builder.testing import BUILDER_LAYER
from ftw.builder.testing import functional_session_factory
from ftw.builder.testing import set_builder_session_factory
from plone.app.testing import FunctionalTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from zope.configuration import xmlconfig


class AvatarLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, BUILDER_LAYER)

    def setUpZope(self, app, configurationContext):
        import ftw.avatar
        xmlconfig.file('configure.zcml',
                       ftw.avatar,
                       context=configurationContext)


AVATAR_FIXTURE = AvatarLayer()
AVATAR_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(AVATAR_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="ftw.avatar:functional")



from OFS.Image import Pdata
from plone.scale.scale import scaleImage
from plone.scale.storage import AnnotationStorage
from Products.Five.browser import BrowserView
from webdav.common import rfc1123_date
from zope.annotation import IAttributeAnnotatable
from zope.interface import alsoProvides


class PortraitScalingView(BrowserView):

    def __call__(self):
        form = self.request.form
        size = form.get('size', form.get('s', None))
        if size is None:
            # return original - no scaling required
            return self.context.index_html(self.request, self.request.RESPONSE)
        else:
            size = int(size)

        if not IAttributeAnnotatable.providedBy(self.context):
            alsoProvides(self.context, IAttributeAnnotatable)

        storage = AnnotationStorage(self.context)
        scale = storage.scale(self.scale_factory,
                              width=size,
                              height=size)

        response = self.request.RESPONSE
        response.setHeader('Last-Modified', rfc1123_date(scale['modified']))
        response.setHeader('Content-Type', scale['mimetype'])
        response.setHeader('Content-Length', len(scale['data']))
        response.setHeader('Accept-Ranges', 'bytes')
        return scale['data']

    def scale_factory(self, **parameters):
        portrait = self.context.data
        if isinstance(portrait, Pdata):
            portrait = str(portrait)
        return scaleImage(portrait, **parameters)



from ftw.avatar.interfaces import IAvatarGenerator
from ftw.avatar.testing import AVATAR_FUNCTIONAL_TESTING
from ftw.avatar.utils import SwitchedToSystemUser
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browser
from ftw.testbrowser import browsing
from OFS.Image import Pdata
from PIL import Image
from Products.CMFCore.utils import getToolByName
from Products.PlonePAS.events import UserLoggedInEvent
from StringIO import StringIO
from unittest2 import TestCase
from zope.component import getUtility
from zope.event import notify
import hashlib
import transaction


def image_size():
    return Image.open(StringIO(browser.contents)).size


def image_hash():
    return hashlib.md5(browser.contents).hexdigest()


class TestPortraitScalingView(TestCase):

    layer = AVATAR_FUNCTIONAL_TESTING

    def setUp(self):
        self.hugo = create(Builder('user').named('Hugo', 'Boss'))
        notify(UserLoggedInEvent(self.hugo))
        self.mtool = getToolByName(self.layer['portal'], 'portal_membership')
        self.portrait = self.mtool.getPersonalPortrait(self.hugo.getId())
        self.portrait_url = self.portrait.absolute_url()
        transaction.commit()  # portrait was generated

    @browsing
    def test_default_scaling_is_original_size(self, browser):
        browser.visit(self.portrait_url)
        self.assertEquals((220, 220), image_size())

    @browsing
    def test_scaling_with_s_parameter(self, browser):
        browser.visit(self.portrait_url + '?s=100')
        self.assertEquals((100, 100), image_size())

    @browsing
    def test_scaling_with_size_parameter(self, browser):
        browser.visit(self.portrait_url + '?size=150')
        self.assertEquals((150, 150), image_size())

    @browsing
    def test_max_scaling_returns_original(self, browser):
        browser.visit(self.portrait_url + '?s=9999')
        self.assertEquals((220, 220), image_size())

    @browsing
    def test_scaling_is_changed_when_image_changes(self, browser):
        browser.visit(self.portrait_url + '?s=100')
        hash_before = image_hash()
        self.regenerate_portrait()
        browser.visit(self.portrait_url + '?s=100')
        hash_after = image_hash()
        self.assertNotEquals(
            hash_before, hash_after,
            'Image scaling cache is not cleared when changing portrait.')

    @browsing
    def test_pdata_portrait_scaling(self, browser):
        """Some files are stored as Pdata objects instead of strings.
        """
        getToolByName(self.layer['portal'], 'portal_memberdata')._setPortrait(
            Pdata(self.generate_portrait().getvalue()), self.hugo.getId())
        transaction.commit()

        browser.visit(self.portrait_url + '?s=100')
        self.assertTrue(image_hash())

    def regenerate_portrait(self):
        portrait = self.generate_portrait()
        with SwitchedToSystemUser():
            self.mtool.changeMemberPortrait(portrait, self.hugo.getId())
        transaction.commit()

    def generate_portrait(self):
        portrait = StringIO()
        getUtility(IAvatarGenerator).generate('AB', portrait)
        portrait.seek(0)
        setattr(portrait, 'filename', 'default.png')
        return portrait



from ftw.avatar.interfaces import IAvatarGenerator
from ftw.avatar import LOGGER
from PIL import Image
from PIL import ImageDraw
from PIL import ImageFont
from Products.CMFCore.utils import getToolByName
from random import random
from zope.component.hooks import getSite
from zope.interface import implements
import os.path


from PIL.ImageFont import _imagingft_not_installed
from PIL.ImageFont import core
if core.__class__ is _imagingft_not_installed:
    FREETYPE_MISSING = (
        'The "_imagingft" C module is not installed, '
        ' which is part of "freetype".'
        ' Install the freetype library and reinstall Pillow with'
        ' freetype support.'
        ' The avatar generation is disabled now.')
    LOGGER.error(FREETYPE_MISSING)
else:
    FREETYPE_MISSING = False


class DefaultAvatarGenerator(object):
    implements(IAvatarGenerator)

    # The resulting image size (square_size * square_size)
    square_size = 220

    def generate(self, userid, output_stream):
        if FREETYPE_MISSING:
            return False
        image = Image.new('RGBA', (self.square_size, self.square_size),
                          self.background_color())
        self.draw_text(image, self.text(userid), self.font())
        image.save(output_stream, 'PNG')
        return True

    def get_name_of_user(self, userid):
        portal = getSite()
        membership = getToolByName(portal, 'portal_membership')
        member = membership.getMemberById(userid)
        if not member:
            return userid
        return member.getProperty('fullname', None) or userid

    def text(self, userid):
        """Returns the text to draw.
        """
        if not userid:
            return '-'

        name = self.get_name_of_user(userid)

        words = name.strip().split(' ', 1)
        if len(words) == 1:
            return words[0][:2].upper()
        else:
            return ''.join([word[0] for word in words]).upper()

    def background_color(self):
        """Pick a random color background color.
        The background color should not be too bright, since the foreground
        font color defaults to white.
        """
        while True:
            red = int(random() * 150) + 50
            green = int(random() * 150) + 50
            blue = int(random() * 150) + 50
            color = red, green, blue

            if max(color) - min(color) > 30:
                return color

    def foreground_color(self):
        """Returns the font color, which is white by default.
        """
        return (255, 255, 255)

    def font(self):
        """Return a PIL ImageFont instance.
        """
        path = os.path.join(os.path.dirname(__file__), 'font',
                            'FantasqueSansMono-Bold.ttf')
        size = int(self.square_size * 0.8)
        return ImageFont.truetype(path, size=size)

    def draw_text(self, image, text, font):
        """Draw the text on the image.
        """
        draw = ImageDraw.Draw(image)
        draw.text(self.text_position(text, font),
                  text,
                  fill=self.foreground_color(),
                  font=font)

    def text_position(self, text, font):
        """Return the top-left point where the text should be placed.
        """
        textwidth, textheight = font.getsize(text)
        left = ((self.square_size - textwidth) / 2.0)
        top = (self.square_size - textheight) / (2 * 1.2)
        return left, top



import AccessControl


class SwitchedToSystemUser(object):
    """Context manager for switching temporarily to a system user.
    """

    def __init__(self):
        self._original_security = None

    def __enter__(self):
        assert self._original_security is None, \
            'Context manager is not reusable'
        self._original_security = AccessControl.getSecurityManager()
        _system_user = AccessControl.SecurityManagement.SpecialUsers.system
        AccessControl.SecurityManagement.newSecurityManager(
            None, _system_user)

    def __exit__(self, _exc_type, _exc_value, _traceback):
        AccessControl.SecurityManagement.setSecurityManager(
            self._original_security)
        self._original_security = None



from zope.interface import Interface


class IAvatarGenerator(Interface):
    """Adapter for generating a default avatar.
    """

    def generate(userid, output_stream):
        """Generates a default avatar image for a user with the passed
        ``userid`` and writes to the ``output_stream``, which is expected
        to be a file-like object.
        """



from ftw.avatar.testing import AVATAR_FUNCTIONAL_TESTING
from unittest2 import TestCase


class TestAutomaticalAvatarGeneration(TestCase):
    layer = AVATAR_FUNCTIONAL_TESTING

    def test_scale_size_is_patched(self):
        from Products.PlonePAS import utils
        self.assertDictContainsSubset({'scale': (300, 300)},
                                      utils.IMAGE_SCALE_PARAMS)



from ftw.avatar import LOGGER

MEMBER_IMAGE_SCALE = (300, 300)


def apply_patches():
    apply_member_image_scale_patch()


def apply_member_image_scale_patch():
    LOGGER.info('Patching maximum member scale in Products.PlonePAS.config'
                ' to %s' % str(MEMBER_IMAGE_SCALE))
    from Products.PlonePAS import config
    config.MEMBER_IMAGE_SCALE = MEMBER_IMAGE_SCALE
    config.IMAGE_SCALE_PARAMS['scale'] = MEMBER_IMAGE_SCALE



from ftw.avatar.member import create_default_avatar
from ftw.avatar.member import get_user_id
from ftw.avatar.testing import AVATAR_FUNCTIONAL_TESTING
from ftw.builder import Builder
from ftw.builder import create
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from Products.CMFCore.utils import getToolByName
from unittest2 import TestCase
import hashlib


class TestCreateDefaultAvatar(TestCase):
    layer = AVATAR_FUNCTIONAL_TESTING

    def test_get_user_id_by_username(self):
        self.assertEquals(TEST_USER_ID, get_user_id(TEST_USER_NAME))

    def test_get_user_id_of_unkown_user(self):
        self.assertEquals(None, get_user_id('unkown'))

    def test_creates_default_avatar_for_users_without_avatar(self):
        hugo = create(Builder('user').named('Hugo', 'Boss'))
        self.assertEquals(None, self.portrait_md5(hugo.getId()))
        create_default_avatar(hugo.getId())
        self.assertTrue(self.portrait_md5(hugo.getId()))

    def test_does_not_override_existing_avatars(self):
        hugo = create(Builder('user').named('Hugo', 'Boss'))
        create_default_avatar(hugo.getId())
        first_hash = self.portrait_md5(hugo.getId())
        create_default_avatar(hugo.getId())
        second_hash = self.portrait_md5(hugo.getId())
        self.assertEquals(first_hash, second_hash,
                          'Existing portrait was overriden by default'
                          ' portrait generator.')

    def portrait_md5(self, userid):
        mship = getToolByName(self.layer['portal'], 'portal_membership')
        mdata = getToolByName(self.layer['portal'], 'portal_memberdata')
        safe_id = mship._getSafeMemberId(userid)
        portrait = mdata._getPortrait(safe_id)
        if not portrait:
            return None

        return hashlib.md5(portrait.data).hexdigest()



from ftw.avatar.testing import AVATAR_FUNCTIONAL_TESTING
from ftw.builder import Builder
from ftw.builder import create
from Products.CMFCore.utils import getToolByName
from Products.PlonePAS.events import UserLoggedInEvent
from unittest2 import TestCase
from zope.event import notify


class TestAutomaticalAvatarGeneration(TestCase):
    layer = AVATAR_FUNCTIONAL_TESTING

    def test_avatar_is_generated_for_new_users_on_login(self):
        hugo = create(Builder('user').named('Hugo', 'Boss'))
        mtool = getToolByName(self.layer['portal'], 'portal_membership')
        self.assertEquals(
            'http://nohost/plone/defaultUser.png',
            mtool.getPersonalPortrait(hugo.getId()).absolute_url())

        notify(UserLoggedInEvent(hugo))

        self.assertEquals(
            'http://nohost/plone/portal_memberdata/portraits/hugo.boss',
            mtool.getPersonalPortrait(hugo.getId()).absolute_url())

    def test_effective_default_avatar_size(self):
        # When setting the portrait, some scaling is involed.
        # Verify that we actually have a big avatar available.
        hugo = create(Builder('user').named('Hugo', 'Boss'))
        notify(UserLoggedInEvent(hugo))

        mtool = getToolByName(self.layer['portal'], 'portal_membership')
        portrait = mtool.getPersonalPortrait(hugo.getId())
        self.assertEquals((220, 220),
                          (portrait.width, portrait.height))



from ftw.avatar.interfaces import IAvatarGenerator
from ftw.avatar.utils import SwitchedToSystemUser
from Products.CMFCore.utils import getToolByName
from StringIO import StringIO
from zope.component.hooks import getSite
from zope.component import getUtility


def get_user_id(userid):
    portal = getSite()
    membership = getToolByName(portal, 'portal_membership')
    member = membership.getMemberById(userid)

    if member:
        return member.getId()

    acl_users = getToolByName(portal, 'acl_users')
    user = acl_users.getUser(userid)
    if user:
        return user.getId()

    return None


def user_has_portrait(userid):
    portal = getSite()
    membership = getToolByName(portal, 'portal_membership')
    memberdata = getToolByName(portal, 'portal_memberdata')
    safe_id = membership._getSafeMemberId(userid)
    return memberdata._getPortrait(safe_id) is not None


def create_default_avatar(userid):
    userid = get_user_id(userid)
    if user_has_portrait(userid):
        return

    portrait = StringIO()
    generator = getUtility(IAvatarGenerator)
    if not generator.generate(userid, portrait):
        return
    portrait.seek(0)
    setattr(portrait, 'filename', 'default.png')

    portal = getSite()
    membership = getToolByName(portal, 'portal_membership')
    with SwitchedToSystemUser():
        membership.changeMemberPortrait(portrait, id=userid)



import os
from setuptools import setup, find_packages


version = '2.1.0'


tests_require = [
    'ftw.builder',
    'ftw.testbrowser',
    'plone.app.testing',
    'transaction',
    'unittest2',
    'zope.configuration',
    ]


setup(name='ftw.avatar',
      version=version,
      description='Generates a personal default avatar for new Plone users',

      long_description=open('README.rst').read() + '\n' + \
          open(os.path.join('docs', 'HISTORY.txt')).read(),

      classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.2',
        'Framework :: Plone :: 4.3',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],

      keywords='ftw avatar personal user portrait plone',
      author='4teamwork AG',
      author_email='mailto:info@4teamwork.ch',
      url='https://github.com/4teamwork/ftw.avatar',

      license='GPL2',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw', ],
      include_package_data=True,
      zip_safe=False,

      install_requires=[
        'AccessControl',
        'Pillow',
        'Products.CMFCore',
        'Products.PlonePAS',
        'Zope2',
        'plone.scale',
        'setuptools',
        'zope.annotation',
        'zope.component',
        'zope.interface',
        'Plone',
        ],

      tests_require=tests_require,
      extras_require=dict(tests=tests_require),

      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



from ftw.avatar.interfaces import IAvatarGenerator
from ftw.avatar.testing import AVATAR_FUNCTIONAL_TESTING
from unittest2 import TestCase
from zope.component import queryUtility
from zope.interface.verify import verifyObject


class TestDefaultAvatarGeneratorUtility(TestCase):

    layer = AVATAR_FUNCTIONAL_TESTING

    def test_utility_is_registered(self):
        self.assertTrue(queryUtility(IAvatarGenerator),
                        'IAvatarGenerator utility is not registered.')

    def test_component_implements_interface(self):
        utility = queryUtility(IAvatarGenerator)
        self.assertTrue(IAvatarGenerator.providedBy(utility))
        verifyObject(IAvatarGenerator, utility)
